<?php
namespace Ig\IgQrcode\Widgets;

use Ig\IgQrcode\Widgets\Provider\ChartDataProvider;
use TYPO3\CMS\Dashboard\WidgetApi;
use TYPO3\CMS\Dashboard\Widgets\AdditionalCssInterface;
use TYPO3\CMS\Dashboard\Widgets\ButtonProviderInterface;
use TYPO3\CMS\Dashboard\Widgets\EventDataInterface;
use TYPO3\CMS\Dashboard\Widgets\RequireJsModuleInterface;
use TYPO3\CMS\Dashboard\Widgets\WidgetConfigurationInterface;
use TYPO3\CMS\Dashboard\Widgets\WidgetInterface;
use TYPO3\CMS\Fluid\View\StandaloneView;

class ChartWidget implements WidgetInterface, EventDataInterface, AdditionalCssInterface, RequireJsModuleInterface
{
    /**
     * @var WidgetConfigurationInterface
     */
    private $configuration;

    /**
     * @var StandaloneView
     */
    private $view;

    /**
     * @var ChartDataProvider
     */
    private $dataProvider;

    /**
     * @var array
     */
    private $options;

    /**
     * @var ButtonProviderInterface|null
     */
    private $buttonProvider;

    /**
     * @var array
     */
    private $chartColor;

    public function __construct(
        WidgetConfigurationInterface $configuration,
        ChartDataProvider $dataProvider,
        StandaloneView $view,
        ButtonProviderInterface $buttonProvider = null,
        array $options = []
    ) {
        $this->configuration = $configuration;
        $this->view = $view;
        $this->dataProvider = $dataProvider;
        $this->options = $options;
        $this->buttonProvider = $buttonProvider;

        // \TYPO3\CMS\Extbase\Utility\DebuggerUtility::var_dump($this->getEventData());
    }

    public function renderWidgetContent(): string
    {
        $this->view->setTemplate($this->options['template']);
        $this->view->assignMultiple([
            'options' => $this->options,
            'configuration' => $this->configuration,
        ]);
        return $this->view->render();
    }
    public function getEventData(): array
    {
        $findByFunc = 'findBy' . ucfirst($this->options['findBy']);
        $chartFunc = $this->options['type'] . 'Chart';

        // check if chart type function is defined
        if (method_exists($this, $chartFunc)) {
            $baseConfig = $this->$chartFunc($findByFunc);
            // check if provider function findBy#### is defined
            if (method_exists($this->dataProvider, $findByFunc)) {
                $baseConfig['graphConfig']['data'] = $this->dataProvider->$findByFunc();
            }
            $baseConfig['graphConfig']['data']['datasets'][0]['label'] = $GLOBALS['LANG']->sL($this->options['label']);
            $baseConfig['graphConfig']['data']['datasets'][0]['backgroundColor'] = $this->chartColors;
            if($this->options['type'] == 'line'){
                $baseConfig['graphConfig']['data']['datasets'][0]['tension'] = 0;
                $baseConfig['graphConfig']['data']['datasets'][0]['fill'] = false;
                $baseConfig['graphConfig']['data']['datasets'][0]['borderColor'] = WidgetApi::getDefaultChartColors()[0];
            }
            return $baseConfig;
        }
    }

    public function lineChart($findBy)
    {
        $this->chartColors = WidgetApi::getDefaultChartColors()[0];
        return [
            'graphConfig' => [
                'type' => 'line',
                'options' => [
                    'maintainAspectRatio' => false,
                    'legend' => [
                        'display' => false,
                    ],
                ],
                'data' => $data,
            ],
        ];
    }

    public function barChart($findBy)
    {
        $this->chartColors = WidgetApi::getDefaultChartColors()[0];
        return [
            'graphConfig' => [
                'type' => 'bar',
                'options' => [
                    'maintainAspectRatio' => false,
                    'legend' => [
                        'display' => false,
                    ],
                    'scales' => [
                        'yAxes' => [
                            [
                                'ticks' => [
                                    'beginAtZero' => true,
                                ],
                            ],
                        ],
                        'xAxes' => [
                            [
                                'ticks' => [
                                    'maxTicksLimit' => 15,
                                ],
                            ],
                        ],
                    ],
                ],
                'data' => $data,
            ],
        ];
    }
    public function doughnutChart($findBy)
    {
        $this->chartColors = WidgetApi::getDefaultChartColors();
        return [
            'graphConfig' => [
                'type' => 'doughnut',
                'options' => [
                    'maintainAspectRatio' => false,
                    'legend' => [
                        'display' => true,
                        'position' => 'bottom',
                    ],
                    'cutoutPercentage' => 60,
                ],
            ],
        ];
    }
    public function getCssFiles(): array
    {
        return ['EXT:dashboard/Resources/Public/Css/Contrib/chart.css'];
    }

    public function getRequireJsModules(): array
    {
        return [
            'TYPO3/CMS/Dashboard/Contrib/chartjs',
            'TYPO3/CMS/Dashboard/ChartInitializer',
        ];
    }

    public function getOptions(): array
    {
        return $this->options;
    }

}
